@@ -28,6 +28,7 @@ from pysnmp import debug
 forceIndexBuild = False
 validateData = False
 v2cArch = False
+v3Only = False
 v3User = 'simulator'
 v3AuthKey = 'auctoritas'
 v3PrivKey = 'privatus'
@@ -36,11 +37,11 @@ agentAddress = ('127.0.0.1', 161)
 deviceDir = '.'
 deviceExt = os.path.extsep + 'snmpwalk'
 
-helpMessage = 'Usage: %s [--help] [--debug=<category>] [--device-dir=<dir>] [--force-index-rebuild] [--validate-device-data] [--agent-address=<X.X.X.X>] [--agent-port=<port>] [--v2c-arch ] [--v3-user=<username>] [--v3-auth-key=<key>] [--v3-priv-key=<key>] [--v3-priv-proto=<DES|AES>]' % sys.argv[0]
+helpMessage = 'Usage: %s [--help] [--debug=<category>] [--device-dir=<dir>] [--force-index-rebuild] [--validate-device-data] [--agent-address=<X.X.X.X>] [--agent-port=<port>] [--v2c-arch ] [--v3-only] [--v3-user=<username>] [--v3-auth-key=<key>] [--v3-priv-key=<key>] [--v3-priv-proto=<DES|AES>]' % sys.argv[0]
 
 try:
     opts, params = getopt.getopt(sys.argv[1:], 'h',
-        ['help', 'debug=', 'device-dir=', 'force-index-rebuild', 'validate-device-data', 'agent-address=', 'agent-port=', 'v2c-arch', 'v3-user=', 'v3-auth-key=', 'v3-priv-key=', 'v3-priv-proto=']
+        ['help', 'debug=', 'device-dir=', 'force-index-rebuild', 'validate-device-data', 'agent-address=', 'agent-port=', 'v2c-arch', 'v3-only', 'v3-user=', 'v3-auth-key=', 'v3-priv-key=', 'v3-priv-proto=']
         )
 except Exception, why:
     print why
@@ -55,27 +56,29 @@ for opt in opts:
     if opt[0] == '-h' or opt[0] == '--help':
         print helpMessage
         sys.exit(-1)
-    if opt[0] == '--debug':
+    elif opt[0] == '--debug':
         debug.setLogger(debug.Debug(opt[1]))
-    if opt[0] == '--device-dir':
+    elif opt[0] == '--device-dir':
         deviceDir = opt[1]
-    if opt[0] == '--force-index-rebuild':
+    elif opt[0] == '--force-index-rebuild':
         forceIndexBuild = True
-    if opt[0] == '--validate-device-data':
+    elif opt[0] == '--validate-device-data':
         validateData = True
-    if opt[0] == '--agent-address':
+    elif opt[0] == '--agent-address':
         agentAddress = (opt[1], agentAddress[1])
-    if opt[0] == '--agent-port':
+    elif opt[0] == '--agent-port':
         agentAddress = (agentAddress[0], string.atoi(opt[1]))
-    if opt[0] == '--v2c-arch':
+    elif opt[0] == '--v2c-arch':
         v2cArch = True
-    if opt[0] == '--v3-user':
+    elif opt[0] == '--v3-only':
+        v3Only = True
+    elif opt[0] == '--v3-user':
         v3User = opt[1]
-    if opt[0] == '--v3-auth-key':
+    elif opt[0] == '--v3-auth-key':
         v3AuthKey = opt[1]
-    if opt[0] == '--v3-priv-key':
+    elif opt[0] == '--v3-priv-key':
         v3PrivKey = opt[1]
-    if opt[0] == '--v3-priv-proto':
+    elif opt[0] == '--v3-priv-proto':
         if opt[1].upper() == 'DES':
             v3PrivProto = config.usmDESPrivProtocol
         elif opt[1].upper() == 'AES':
@@ -322,16 +325,15 @@ class DeviceFile:
 
 # Collect device files
 
-def getDevices(topDir, dirDepth=None):
-    if dirDepth is None:
-        dirDepth = len(topDir.split(os.path.sep))
-    files = os.listdir(topDir)
+def getDevices(tgtDir, topLen=None):
+    if topLen is None:
+        topLen = len(tgtDir.split(os.path.sep))
     dirContent = []
-    for dFile in files:
-        path = topDir + os.path.sep + dFile
-        inode = os.stat(path)
+    for dFile in os.listdir(tgtDir):
+        fullPath = tgtDir + os.path.sep + dFile
+        inode = os.stat(fullPath)
         if inode[0] & 0x4000:
-            dirContent = dirContent + getDevices(path, dirDepth)
+            dirContent = dirContent + getDevices(fullPath, topLen)
             continue            
         if not (inode[0] & 0x8000):
             continue
@@ -341,12 +343,10 @@ def getDevices(topDir, dirDepth=None):
             continue
         if not parserSet.has_key(dExt):
             continue
-        fullPath = topDir + os.path.sep + dFile
-        contextName = fullPath.split(os.path.sep)[dirDepth:]
+        fullPath = tgtDir + os.path.sep + dFile
+        relPath = fullPath.split(os.path.sep)[topLen:]
         dirContent.append(
-            (fullPath,
-             parserSet[dExt],
-             os.path.sep.join(contextName)[:-len(dExt)])
+            (fullPath, parserSet[dExt], os.path.sep.join(relPath)[:-len(dExt)])
             )
     return dirContent
 
@@ -455,9 +455,10 @@ class MibInstrumController:
 # Devices index as a MIB instrumentaion at a dedicated SNMP context
 
 class DevicesIndexInstrumController:
-    def __init__(self, baseOid=(1, 3, 6, 1, 4, 1, 20408, 999, 1)):
+    indexSubOid = (1,)
+    def __init__(self, baseOid=(1, 3, 6, 1, 4, 1, 20408, 999)):
         self.__db = indices.OidOrderedDict()
-        self.__baseOid = baseOid
+        self.__indexOid = baseOid + self.indexSubOid
         self.__idx = 1
 
     def readVars(self, varBinds, (acFun, acCtx)=(None, None)):
@@ -478,16 +479,16 @@ class DevicesIndexInstrumController:
         return map(
             lambda (o,v): self.__getNextVal(o, exval.endOfMib), varBinds
             )
-    
+
     def writeVars(self, varBinds, (acFun, acCtx)=(None, None)):
         return map(
             lambda (o,v): (o, exval.noSuchInstance), varBinds
             )
-
+    
     def addDevice(self, *args):
         for idx in range(len(args)):
             self.__db[
-                self.__baseOid + (idx+1, self.__idx)
+                self.__indexOid + (idx+1, self.__idx)
                 ] = rfc1902.OctetString(args[idx])
         self.__idx = self.__idx + 1
 
@@ -509,7 +510,7 @@ else:
         config.usmHMACMD5AuthProtocol, v3AuthKey,
         v3PrivProto, v3PrivKey
         )
-
+        
 # Build pysnmp Managed Objects base from device files information
 
 for fullPath, textParser, communityName in getDevices(deviceDir):
@@ -519,30 +520,30 @@ for fullPath, textParser, communityName in getDevices(deviceDir):
 
     print """Device %s
 SNMPv1/2c community name: %s""" % (mibInstrum, communityName)
-    
+
     if v2cArch:
         contexts[communityName] = mibInstrum
-
+        
         devicesIndexInstrumController.addDevice(
             fullPath, communityName
             )
     else:
         agentName = contextName = hashlib.md5(communityName).hexdigest()
+
+        if not v3Only:
+            config.addV1System(
+                snmpEngine, agentName, communityName, contextName=contextName
+                )
         
-        config.addV1System(snmpEngine, agentName,
-                           communityName, contextName=contextName)
-
-        snmpContext.registerContextName(
-            contextName, mibInstrum
-            )
-
+        snmpContext.registerContextName(contextName, mibInstrum)
+                 
         devicesIndexInstrumController.addDevice(
             fullPath, communityName, contextName
             )
-
+                 
         print """SNMPv3 context name: %s
 """ % (contextName,)
-
+        
 if v2cArch:
     def getBulkHandler(varBinds, nonRepeaters, maxRepetitions, readNextVars):
         if nonRepeaters < 0: nonRepeaters = 0
